@@ -17,11 +17,14 @@ from generic_utils.models import GenericModelBase
 
 class BaseImageModel(models.Model):
     ''' Simple abstract Model class with image field.
+    
+        .. attribute:: image
+        
+            ``models.ImageField``
     '''
     
     def get_upload_path(self, filename):
-        ''' Override this to customize upload path
-        '''
+        ''' Override this to customize upload path '''
         raise NotImplementedError
 
     def _upload_path_wrapper(self, filename):
@@ -66,16 +69,34 @@ class AbstractAttachedImage(ReplaceOldImageModel, GenericModelBase):
     '''
         Abstract Image model that can be attached to any other Django model using 
         generic relations.
-    '''    
+        
+        .. attribute:: is_main
+        
+            BooleanField. Whether the image is the main image for object. 
+            This field is set to False automatically for all images attached to 
+            same object if image with is_main=True is saved to ensure that there 
+            is only 1 main image for object.
+            
+        .. attribute:: order
+        
+            IntegerField to support ordered image sets. 
+            On creation it is set to max(id)+1.
+            
+    '''
+    
     
     user = models.ForeignKey(User, blank=True, null=True, verbose_name=_('User'))
+    'A ForeignKey to associated user, for example user who uploaded image. Can be empty.'
     
     caption = models.TextField(_('Caption'), null=True, blank=True)
+    'TextField caption for image'
+    
     is_main = models.BooleanField(_('Main image'), default=False)
     
     order = models.IntegerField(_('Order'), default=0)
 
-    objects = AttachedImageManager()          
+    objects = AttachedImageManager()   
+    'Default manager of :class:`~generic_images.managers.AttachedImageManager` type.'
     
     def next(self):
         ''' Returns next image for same content_object and None if image is the last. '''
@@ -100,7 +121,8 @@ class AbstractAttachedImage(ReplaceOldImageModel, GenericModelBase):
     def get_file_name(self, filename):        
         ''' Returns file name (without path and extenstion) 
             for uploaded image. Default is 'max(pk)+1'. 
-            Override this in subclass if you want different file names.
+            Override this in subclass or assign another functions per-instance 
+            if you want different file names (ex: random string).
         '''        
 #        alphabet = "1234567890abcdefghijklmnopqrstuvwxyz"        
 #        return ''.join([random.choice(alphabet) for i in xrange(16)]) # 1e25 variants
@@ -109,9 +131,10 @@ class AbstractAttachedImage(ReplaceOldImageModel, GenericModelBase):
         
     def get_upload_path(self, filename):
         ''' Override this in proxy subclass to customize upload path.
-            Default upload path is ``/media/images/<user.id>/<filename>.<ext>``
-            or ``/media/images/common/<filename>.<ext>`` if user is not set.
-            filename is returned by get_file_name method.
+            Default upload path is :file:`/media/images/<user.id>/<filename>.<ext>`
+            or :file:`/media/images/common/<filename>.<ext>` if user is not set.
+            
+            filename is returned by :meth:`~generic_images.models.AbstractAttachedImage.get_file_name` method.
             By default it is probable id of new image (it is
             predicted as it is unknown at this stage).
         '''        
@@ -121,19 +144,27 @@ class AbstractAttachedImage(ReplaceOldImageModel, GenericModelBase):
         return os.path.join('media', 'images', user_folder, self.get_file_name(filename) + ext)    
     
     
-    def save(self, *args, **kwargs):        
+    def save(self, *args, **kwargs):
+        send_signal = getattr(self, 'send_signal', True)
         if self.is_main:
             related_images = self.__class__.objects.filter(content_type=self.content_type, 
                                                           object_id=self.object_id)
             related_images.update(is_main=False)
-        if not self.pk:
-            self.order = self._get_next_pk()
-        super(AbstractAttachedImage, self).save(*args, **kwargs)         
-        image_saved.send(sender = self.content_type.model_class(), instance = self)
+            
+        if not self.pk: # object is created
+            if not self.order: # order is not set
+                self.order = self._get_next_pk() # let it be max(pk)+1
+                
+        super(AbstractAttachedImage, self).save(*args, **kwargs)
+        
+        if send_signal: 
+            image_saved.send(sender = self.content_type.model_class(), instance = self)
         
         
     def delete(self, *args, **kwargs):
-        image_deleted.send(sender = self.content_type.model_class(), instance = self)
+        send_signal = getattr(self, 'send_signal', True)        
+        if send_signal:
+            image_deleted.send(sender = self.content_type.model_class(), instance = self)
         super(AbstractAttachedImage, self).delete(*args, **kwargs)            
         
         
@@ -157,7 +188,8 @@ class AbstractAttachedImage(ReplaceOldImageModel, GenericModelBase):
 class AttachedImage(AbstractAttachedImage):
     '''
         Image model that can be attached to any other Django model using 
-        generic relations.
+        generic relations. It is simply non-abstract subclass of 
+        :class:`~generic_images.models.AbstractAttachedImage`
     '''    
     class Meta:
         ordering = ['-order']
