@@ -7,10 +7,85 @@ def _pop_data_from_kwargs(kwargs):
     ct_field = kwargs.pop('ct_field', 'content_type')
     fk_field = kwargs.pop('fk_field', 'object_id')
     return ct_field, fk_field
+
+
+class RelatedInjector(models.Manager):
+    """ Manager that can emulate ``select_related`` fetching 
+        reverse relations using 1 additional SQL query.
+    """
+    def __init__(self, fk_field='object_id', *args, **kwargs):
+        self.fk_field = fk_field
+        super(RelatedInjector, self).__init__(*args, **kwargs)
+        
+    def inject_to(self, objects, field_name, get_inject_object = lambda obj: obj,  **kwargs):
+        '''        
+        ``objects`` is an iterable. Related objects 
+            will be attached to elements of this iterable.
+            
+        ``field_name`` is the attached object attribute name
+        
+        ``get_injector_object`` is a callable that takes object in `objects` 
+            iterable. Related objects will be available as an attribute of the 
+            result of ``get_inject_object(obj)``. It is assumed that ``fk_field`` 
+            points to  ``get_inject_object(obj)``.
+        
+        All other kwargs will be passed as arguments to queryset filter function.
+        
+        For example, we need to prefetch user profiles when we display a list of 
+        comments::
+        
+            # models.py
+            class UserProfile(models.Model):
+                user = models.ForeignKey(User, unique=True)
+                info = models.CharField(max_length=100)
+                objects = models.Manager()
+                injector = RelatedInjector(fk_field='user')
+                
+            # views.py            
+            def show_comments(request, obj_id):
+                ...
+                comments = list(Comment.objects.for_model(obj).select_related('user'))                                                
+                UserProfile.injector.inject_to(comments, '_profile_cache', 
+                                               lambda comment: comment.user)
+                
+                return direct_to_template('comment_list.html', {'comments': comments})
+             
+            # in comment_list.html             
+            {% for comment in comments %}
+                <h3>{{ comment.user }}</h3>
+                <h4>{{ comment.user.get_profile.info }}</h4>
+                {{ comment.comment|linebreaks }}
+            {% endfor %}
+             
+        ``comment.user`` attribute will be selected using ``select_related`` and
+        ``comment.user._profile_cache`` (exposed by get_profile method) will be
+        selected by our injector. So there will be only 2 SQL queries for 
+        selecting all comments with users and user profiles.
+        
+        '''
+
+        #get related data
+        kwargs.update({self.fk_field+'__in': [ get_inject_object(obj).pk for obj in objects ]})
+        
+        data = self.get_query_set().filter(**kwargs)
+        data_dict = dict((getattr(item, self.fk_field), item) for item in list(data))
+        
+        # add info to original data
+        for obj in objects:
+            injected_obj = get_inject_object(obj)
+            
+            if data_dict.has_key(injected_obj): 
+                # fk_field was ForeignKey so there are objects in lookup dict
+                get_inject_object(obj).__setattr__(field_name, data_dict[injected_obj])
+                
+            elif data_dict.has_key(injected_obj.pk): 
+                # fk_field was simple IntegerField so there are pk's in lookup dict
+                get_inject_object(obj).__setattr__(field_name, data_dict[injected_obj.pk])
     
 
-class GenericInjector(models.Manager):
-    ''' Manager for selecting all generic-related objects in one (two) SQL queries.
+class GenericInjector(RelatedInjector):
+    ''' RelatedInjector but for GenericForeignKey's. 
+        Manager for selecting all generic-related objects in one (two) SQL queries.
         Selection is performed for a list of objects. Resulting data is aviable as attribute 
         of original model. Only one instance per object can be selected. Example usage: 
         select (and make acessible as user.avatar) all avatars for a list of user when 
@@ -34,15 +109,15 @@ class GenericInjector(models.Manager):
             2. one query for selecting all avatars (images with is_main=True) for selected users
             3. and maybe one query for selecting content-type for User model
 
-One can reuse GenericInjector manager for other models that are supposed to 
-be attached via generic relationship. It can be considered as an addition to 
-GFKmanager and GFKQuerySet from djangosnippets for different use cases.
+        One can reuse GenericInjector manager for other models that are supposed to 
+        be attached via generic relationship. It can be considered as an addition to 
+        GFKmanager and GFKQuerySet from djangosnippets for different use cases.
         
     '''
     
-    def __init__(self, *args, **kwargs):
-        self.ct_field, self.fk_field = _pop_data_from_kwargs(kwargs)    
-        super(GenericInjector, self).__init__(*args, **kwargs)
+    def __init__(self, fk_field='object_id', ct_field='content_type', *args, **kwargs):
+        self.ct_field = ct_field
+        super(GenericInjector, self).__init__(fk_field, *args, **kwargs)
         
     
     def inject_to(self, objects, field_name, get_inject_object = lambda obj: obj, **kwargs):
@@ -52,7 +127,7 @@ GFKmanager and GFKQuerySet from djangosnippets for different use cases.
         
         ``field_name`` is the attached object attribute name
         
-        ``get_injector_object`` is a callable that takes object in `objects` iterable.
+        ``get_inject_object`` is a callable that takes object in `objects` iterable.
             Image will be available as an attribute of the result of 
             `get_injector_object(object)`. Images attached to `get_injector_object(object)`
             will be selected.
@@ -73,20 +148,8 @@ GFKmanager and GFKQuerySet from djangosnippets for different use cases.
         except IndexError:
             return objects
         
-        #get related data
-        kwargs.update({
-                        self.ct_field: content_type, 
-                        self.fk_field+'__in': [ get_inject_object(object).pk for object in objects ]
-                     })
-        data = self.get_query_set().filter(**kwargs)                
-        data_dict = dict((getattr(item, self.fk_field), item) for item in list(data))
-        
-        # add info to original data
-        for object in objects:
-            pk = get_inject_object(object).pk
-            if data_dict.has_key(pk):
-                get_inject_object(object).__setattr__(field_name, data_dict[pk])    
-
+        kwargs.update({self.ct_field: content_type})
+        return super(GenericInjector, self).inject_to(objects, field_name, get_inject_object, **kwargs)
     
     
 class GenericModelManager(models.Manager):
